/**
 * @license
 * Copyright 2019 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
/// <reference types="@webgpu/types" />
import { DataType, Rank, ShapeMap, TensorInfo } from '@tensorflow/tfjs-core';
import { Glslang } from '@webgpu/glslang/dist/web-devel/glslang.onefile';
import * as shader_preprocessor from '../shader_preprocessor';
export interface WebGPUProgram {
    shaderKey?: string;
    userCode: string;
    outputShape: number[];
    dispatchLayout: {
        x: number[];
        y?: number[];
        z?: number[];
    };
    dispatch: [number, number, number];
    variableNames: string[];
    uniforms?: string;
    needsShapesUniforms: boolean;
    workPerThread?: number;
    workGroupSize?: [number, number, number];
}
export interface WebGPUBinary {
    bindGroupLayout: GPUBindGroupLayout;
    pipeline: GPUComputePipeline;
}
export interface TensorData {
    dtype: DataType;
}
export interface BindingInfo {
    resource: {
        offset: number;
        size: number;
        buffer: GPUBuffer;
    };
}
export declare const makeBindGroup: (device: GPUDevice, bindGroupLayout: GPUBindGroupLayout, inputs: BindingInfo[], output: BindingInfo, uniforms?: BindingInfo) => GPUBindGroup;
export declare const compileProgram: (glslang: Glslang, device: GPUDevice, program: WebGPUProgram, inputsData: shader_preprocessor.InputInfo[], output: TensorInfo, uniforms?: BindingInfo) => WebGPUBinary;
export declare function makeShaderKey<R extends Rank>(program: WebGPUProgram, shapes: Array<ShapeMap[R]>, types: string[]): string;
